﻿using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Services.Optimization.Billing.Interfaces;

namespace HeuristicLab.Services.Optimization.Billing.Business {
  public class MockupBillingService : IOptimizationBilling {

    private static IList<Model.Product> products;
    private static IDictionary<string, IList<Model.Order>> userToOrders;
    private static IList<Model.Order> allOrders;
    private static IDictionary<string, Model.User> users;
    private static long maxId = 1;

    static MockupBillingService() {
      products = new List<Model.Product>();
      products.Add(new Model.Product() {
        Name = "Optimization-as-a-Service - Experiment execution",
        Description = "Create and run your experiments within HeuristicLab Hive",
        ProductId = 1,
        Price = 10.0,
        ProductType = "Optimization Service"
      });
      products.Add(new Model.Product() {
        Name = "Whatever-o-matic - Experiment execution",
        Description = "Another fine product",
        ProductId = 2,
        Price = 100.0,
        ProductType = "Product"
      });

      userToOrders = new Dictionary<string, IList<Model.Order>>();
      allOrders = new List<Model.Order>();
      users = new Dictionary<string, Model.User>();
    }

    public IList<Model.Product> GetProducts() {
      return products;
    }

    public void SaveOrder(Model.Order order) {
      IList<Model.Order> orders;
      if (!userToOrders.TryGetValue(order.User.Name, out orders)) {
        userToOrders[order.User.Name] = orders = new List<Model.Order>();        
      }
      var oldOrder = (from o in orders where o.OrderId == order.OrderId select o).FirstOrDefault();
      if (oldOrder == null) {
        order.OrderId = maxId++;
        orders.Add(order);
        allOrders.Add(order);
      }
      else {
        oldOrder.Invoices = order.Invoices;
        oldOrder.OrderLines = order.OrderLines;
        oldOrder.OrderStateId = order.OrderStateId;
        oldOrder.State = order.State;
        oldOrder.ActiveSince = order.ActiveSince;
        oldOrder.ActiveUntil = order.ActiveUntil;
        oldOrder.BillingPeriod = order.BillingPeriod;
        oldOrder.BillingType = order.BillingType;
      }
    }

    public IList<Model.Order> GetOrdersByState(Model.OrderState state) {
      return (from o in allOrders where o.State == state select o).ToList();
    }


    public IList<Model.Order> GetOrders(string userName) {
      return (from o in allOrders where o.User.Name == userName select o).ToList();
    }


    public Model.User GetUser(string userName) {
      Model.User user;
      if (!users.TryGetValue(userName, out user))
        return null;
      return user;
    }

    public void SaveUser(Model.User user) {
      users[user.Name] = user;
    }
  }
}
